@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland <vadim@vk.crocodile.org>
 
-  $Id: Address.cpp,v 1.2 2004/09/08 05:34:50 vkurland Exp $
+  $Id: Address.cpp,v 1.3 2006/03/05 04:15:42 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -48,12 +48,13 @@ void Address::setNetmask(const std::string &nm) {}
 
 FWReference* Address::createRef()
 {
-    FWObjectReference *ref=new FWObjectReference();
+//    FWObjectReference *ref=new FWObjectReference();
+    FWObjectReference *ref=FWObjectReference::cast(getRoot()->create(FWObjectReference::TYPENAME, true));
     ref->setPointer(this);
     return ref;
 }
 
 bool Address::isAny() const
 { 
-    return getId()==FWObjectDatabase::cast(getRoot())->getAnyNetworkId();
+    return getId()==FWObjectDatabase::getAnyNetworkId();
 }
