@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWObject.cpp,v 1.5 2005/03/31 05:49:40 vkurland Exp $
+  $Id: FWObject.cpp,v 1.10 2006/05/17 04:16:34 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -87,12 +87,13 @@ void FWObject::fromXML(xmlNodePtr root) throw(FWException)
     }
 
     ref_counter=0;
+    FWObjectDatabase *dbroot = getRoot();
 
     for(xmlNodePtr cur=root->xmlChildrenNode; cur; cur=cur->next) 
     {
         if(cur && !xmlIsBlankNode(cur))  
         {
-            FWObject *o=FWObjectDatabase::db->createFromXML(cur);
+            FWObject *o=dbroot->createFromXML(cur);
             if(o!=NULL) 
             {
 		add(o);
@@ -126,6 +127,8 @@ xmlNodePtr FWObject::toXML(xmlNodePtr parent, bool process_children) throw(FWExc
     {
         const string &name  = (*i).first;
         const string &value = (*i).second;
+
+        if (name[0]=='.') continue;
         xmlAttrPtr pr = xmlNewProp(me, 
                                    STRTOXMLCAST(name),
                                    STRTOXMLCAST(value));
@@ -151,11 +154,12 @@ FWObject::FWObject()
     init        = false;
     ref_counter = 0;
     parent      = NULL;
+    dbroot      = NULL;
 
     setName("New object");
     
     // When object created we assign it unique Id
-    setId(FWObjectDatabase::db->generateUniqueId());
+    setId(FWObjectDatabase::generateUniqueId());
 
     setDirty(false);
 }
@@ -166,22 +170,17 @@ FWObject::FWObject(const FWObject &c) : list<FWObject*>(c)
     *this=c;
 }
 
-
-/**
- * we do not need to do anything with tree root in FWObject, but
- * derived classes might want to use it to create children objects and
- * add references etc.
- */
 FWObject::FWObject(const FWObject *root)
 {
     init        = false;
     ref_counter = 0    ;
     parent      = NULL ;
+    dbroot      = (FWObject*)root;
     
     setName("New object");
     
     // When object created we assign it unique Id
-    setId(FWObjectDatabase::db->generateUniqueId());
+    setId(FWObjectDatabase::generateUniqueId());
 
     setDirty(false);
 }
@@ -254,7 +253,7 @@ bool FWObject::cmp(const FWObject *obj) throw(FWException)
     for ( ; i1!=end(); ++i1)
     {
         bool found=false;
-        FWObject *o2=NULL;
+//        FWObject *o2=NULL;
         FWObject::const_iterator j1=obj->begin();
         for ( ; j1!=obj->end(); ++j1)
         {
@@ -290,7 +289,7 @@ FWObject& FWObject::operator=(const FWObject &x)  throw(FWException)
 FWObject& FWObject::duplicate(const FWObject *x, bool preserve_id) throw(FWException)
 {
     checkReadOnly();
-
+    bool xro=x->getBool("ro"); 
     shallowDuplicate(x,preserve_id);
 
     destroyChildren();
@@ -302,15 +301,20 @@ FWObject& FWObject::duplicate(const FWObject *x, bool preserve_id) throw(FWExcep
     }
 
     setDirty(true);
+    if (xro) setReadOnly(true);
     return *this;
 }
 
 FWObject* FWObject::addCopyOf(const FWObject *x, bool preserve_id) throw(FWException)
 {
     if (x==NULL) return NULL;
-
-    FWObject *o1=FWObjectDatabase::db->create(x->getTypeName());
-    if(!o1)   throw FWException(string("Error creating object with type: ")+x->getTypeName());
+    FWObject *o1;
+    FWObjectDatabase *root = getRoot();
+    if (root==NULL) root = x->getRoot();
+    o1 = root->create(x->getTypeName());
+    if(!o1)
+        throw FWException(string("Error creating object with type: ")+
+                          x->getTypeName());
     add(o1);
     o1->duplicate(x, preserve_id);
     return o1;
@@ -328,7 +332,7 @@ FWObject& FWObject::shallowDuplicate(const FWObject *x, bool preserve_id) throw(
     string id = getId();
 
     data     = x->data;
-
+    if (getBool("ro")) setReadOnly(false);
     if(!preserve_id)
     {
         ref_counter = 0           ;
@@ -336,6 +340,8 @@ FWObject& FWObject::shallowDuplicate(const FWObject *x, bool preserve_id) throw(
     } else if(id!="")   // some objects do not have ID per DTD (e.g. Src, Dst, etc.)
 	setId(id);
 
+    if (dbroot==NULL) setRoot(x->getRoot());
+
     setDirty(true);
     return *this;
 }
@@ -364,12 +370,15 @@ FWObject*  FWObject::getLibrary() const
     return (FWObject*)p;
 }
 
-FWObject* FWObject::getRoot() const
+FWObjectDatabase* FWObject::getRoot() const
 {
+    return FWObjectDatabase::cast(dbroot);
+#if 0
     const FWObject *p=this;
 
     while(p->getParent()!=NULL) p=p->getParent();
-    return (FWObject*)p;
+    return FWObjectDatabase::cast((FWObject*)p);
+#endif
 }
 
 string FWObject::getPath(bool relative) const
@@ -405,10 +414,15 @@ const string &FWObject::getId() const
     return getStr("id");
 }
 
+/*
+ * need to update index because ID of the object changes
+ */
 void FWObject::setId(const string &c)
 {
     setStr("id",c);
     setDirty(true);
+    if (dbroot!=NULL)
+        FWObjectDatabase::cast(dbroot)->addToIndex(this);
 }
 
 bool FWObject::exists(const string &name) const 
@@ -445,7 +459,7 @@ void FWObject::remStr(const string &name)
 
 void FWObject::setStr(const string &name, const string &val)
 {
-    checkReadOnly();
+    if (name[0]!='.') checkReadOnly();
 
     data[name]=val;
     setDirty(true);
@@ -569,11 +583,22 @@ void FWObject::_adopt(FWObject *obj)
 {
     obj->ref();
     obj->setParent(this);
+/*
+ * normally we always create objects using FWObjectDatabase::create
+ * which sets dbroot in the object. However, if an object is created
+ * just with 'new ...' and then added to the database using
+ * FWObject::add, it doesn't have correct pointer to the database
+ * root. We can easily repair this by setting dbroot here. Even if
+ * this operation is redundant, its cost is very low and it adds
+ * robustness.
+ *
+ */
+    obj->setRoot(getRoot());
 }
 
 void FWObject::addAt(const string& where_id, FWObject *obj)
 {
-    FWObject *p=getById( where_id , true );
+    FWObject *p=getRoot()->findInIndex( where_id );
     assert (p!=NULL);
     p->add(obj);
 }
@@ -592,7 +617,8 @@ void FWObject::add(FWObject *obj,bool validate)
 
 FWReference* FWObject::createRef()
 {
-    FWObjectReference *ref=new FWObjectReference();
+//    FWObjectReference *ref=new FWObjectReference();
+    FWObjectReference *ref=FWObjectReference::cast(getRoot()->create(FWObjectReference::TYPENAME, true));
     ref->setPointer(this);
     return ref;
 }
@@ -610,6 +636,8 @@ void FWObject::addRef(FWObject *obj)
 	push_back(oref);
 	_adopt(oref);
 	setDirty(true);
+// see comment in FWObject::_adopt
+        obj->setRoot(getRoot());
     }
 }
 
@@ -673,8 +701,8 @@ void FWObject::swapObjects(FWObject *o1, FWObject *o2)
  */
 void FWObject::_moveToDeletedObjects(FWObject *obj)
 {
-    FWObjectDatabase *root = FWObjectDatabase::cast(getRoot());
-    FWObject *dobj = root->getById( root->getDeletedObjectsId() );
+    FWObjectDatabase *root = getRoot();
+    FWObject *dobj = root->findInIndex( root->getDeletedObjectsId() );
             
     if (dobj==NULL)
     {
@@ -725,7 +753,6 @@ void FWObject::removeAllInstances(FWObject *rm)
 
     bool deletedObject =
         (rm->getParent()->getId()==FWObjectDatabase::getDeletedObjectsId());
-
     removeAllReferences(rm);
     _removeAll(rm);
 
@@ -936,13 +963,17 @@ FWObjectTypedChildIterator FWObject::findByType(const std::string &type_name) co
  */
 void  FWObject::setDirty(bool f)
 {
-    if (getRoot()==this) dirty=f;
-    else                 getRoot()->setDirty(f);
+    FWObjectDatabase *dbroot = getRoot();
+    if (dbroot==NULL) return;
+    if (dbroot==this) dirty=f;
+    else              dbroot->setDirty(f);
 }
 
 bool FWObject::isDirty()
 {
-    return (getRoot()->dirty);
+    FWObjectDatabase *dbroot = getRoot();
+    if (dbroot==NULL) return false;
+    return (dbroot->dirty);
 }
 
 /*
@@ -952,10 +983,14 @@ bool FWObject::isDirty()
 void FWObject::setReadOnly(bool f)
 {
     data["ro"]= (f)?"True":"False" ;
-    bool ri=getRoot()->init;
-    getRoot()->init=true;
-    setDirty(true);
-    getRoot()->init=ri;
+    FWObjectDatabase *dbroot = getRoot();
+    if (dbroot)
+    {
+        bool ri=dbroot->init;
+        dbroot->init=true;
+        setDirty(true);
+        dbroot->init=ri;
+    }
 //    setBool("ro",f);
 }
 
@@ -970,7 +1005,8 @@ void FWObject::setReadOnly(bool f)
  */
 bool FWObject::isReadOnly()
 {
-    if (getRoot()->init) return false;
+    FWObjectDatabase *dbroot = getRoot();
+    if (dbroot==NULL || dbroot->init) return false;
     FWObject *p=this;
     while (p)
     {
