@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: FWObjectDatabase.cpp,v 1.7 2005/02/02 06:32:26 vkurland Exp $
+  $Id: FWObjectDatabase.cpp,v 1.16 2006/04/23 15:45:25 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -53,6 +53,8 @@
 #include <fwbuilder/Interface.h>
 #include <fwbuilder/IPv4.h>
 #include <fwbuilder/physAddress.h>
+#include <fwbuilder/DNSName.h>
+#include <fwbuilder/AddressTable.h>
 #include <fwbuilder/InterfacePolicy.h>
 #include <fwbuilder/Group.h>
 #include <fwbuilder/Rule.h>
@@ -62,6 +64,7 @@
 #include <fwbuilder/Firewall.h>
 #include <fwbuilder/NAT.h>
 #include <fwbuilder/Policy.h>
+#include <fwbuilder/Routing.h>
 #include <fwbuilder/ObjectGroup.h>
 #include <fwbuilder/ServiceGroup.h>
 #include <fwbuilder/IntervalGroup.h>
@@ -69,6 +72,7 @@
 #include <fwbuilder/AddressRange.h>
 #include <fwbuilder/Management.h>
 #include <fwbuilder/XMLTools.h>
+#include <fwbuilder/TagService.h>
 
 #include <iostream>
 
@@ -82,7 +86,6 @@ IDcounter::IDcounter()
 
 IDcounter          IDc;
 
-FWObjectDatabase*  FWObjectDatabase::db        = NULL;
 const char*        FWObjectDatabase::TYPENAME  = {"FWObjectDatabase"};
 //long               FWObjectDatabase::IDcounter = 0;
 //Mutex FWObjectDatabase::IDmutex;
@@ -91,7 +94,10 @@ const string FWObjectDatabase::DTD_FILE_NAME  = "fwbuilder.dtd"    ;
 
 FWObjectDatabase::FWObjectDatabase():FWObject(), data_file()
 {
-    if(db==NULL) db=this;
+    setRoot(this);
+    index_hits = index_misses = 0;
+
+//    if(db==NULL) db=this;
 
     lastModified = 0;
 
@@ -103,6 +109,9 @@ FWObjectDatabase::FWObjectDatabase():FWObject(), data_file()
 
 FWObjectDatabase::FWObjectDatabase(FWObjectDatabase& d):FWObject()
 {
+    setRoot(this);
+    index_hits = index_misses = 0;
+
     data_file = d.data_file;
 
     setName(TYPENAME);
@@ -162,6 +171,7 @@ void FWObjectDatabase::load(const string &f,
 
 //        clearChildren();
         deleteChildren();
+        clearIndex();
 
         fromXML(root);
         setDirty(false);
@@ -208,6 +218,30 @@ void FWObjectDatabase::saveFile(const string &filename) throw(FWException)
     init=false;
 }
 
+void FWObjectDatabase::saveToBuffer(xmlChar **buffer,int *size) throw(FWException)
+{
+/* need to set flag 'init' so we ignore read-only status. Some objects
+ * modify themselves in toXML (e.g. Management) so if they belong to a
+ * read-only library, we can't save them to a file. It should be safe
+ * to ignore read-only flag but save it though.
+ */
+    init=true;
+
+    xmlDocPtr doc = xmlNewDoc(TOXMLCAST("1.0"));
+  
+    doc->children = xmlNewDocNode(doc, NULL, STRTOXMLCAST(getName()), NULL);
+    xmlNewNs(doc->children, TOXMLCAST("http://www.fwbuilder.org/1.0/"), NULL);
+
+    toXML(xmlDocGetRootElement(doc));
+
+    XMLTools::dumpToMemory(doc, buffer, size, FWObjectDatabase::TYPENAME, FWObjectDatabase::DTD_FILE_NAME);
+
+    xmlFreeDoc(doc);  
+//    setDirty(false);
+
+    init=false;
+}
+
 void FWObjectDatabase::fromXML(xmlNodePtr root) throw(FWException)
 {
     FWObject::fromXML(root);
@@ -277,7 +311,15 @@ string FWObjectDatabase::generateUniqueId()
     // library is not initialized.
 
 //    IDmutex.lock();
-    sprintf(res,"id%lX",IDc.get());
+    int pid=0;
+#ifdef _WIN32
+    pid = _getpid();
+#else
+    pid = getpid();
+#endif
+    
+
+    sprintf(res,"id%lX%d",IDc.get(),pid);
 //    IDc++;
 //    IDmutex.unlock();
     
@@ -290,30 +332,109 @@ void  FWObjectDatabase::setDirty(bool f)
     if(!init && f) lastModified=time(NULL);
 }
 
+void FWObjectDatabase::addToIndex(FWObject* o)
+{
+    if (o)
+    {
+        o->setRoot( this );
+        if (!o->getId().empty() ) obj_index[o->getId()] = o;
+    }
+}
+
+FWObject* FWObjectDatabase::findInIndex(const std::string &id)
+{
+    FWObject *o = obj_index[id];
+    if (o!=NULL) index_hits++;
+    else
+    {
+        index_misses++;
+// if index is incomplete or empty, update it automatically using
+// recursive search to find object
+        o = getById( id , true );
+        if (o) addToIndex(o);
+    }
+    return o;
+}
+
+void FWObjectDatabase::buildIndex()
+{
+    clearIndex();
+    _addToIndexRecursive(this);
+}
+
+void FWObjectDatabase::clearIndex()
+{
+    index_hits = index_misses = 0;
+    obj_index.clear();
+}
+
+void FWObjectDatabase::getIndexStats(int &index_size, int &hit_counter, int &miss_counter)
+{
+    index_size = obj_index.size();
+    hit_counter = index_hits;
+    miss_counter = index_misses;
+}
+
+void FWObjectDatabase::_addToIndexRecursive(FWObject *o)
+{
+    addToIndex(o);
+    for (FWObject::iterator i=o->begin(); i!=o->end(); ++i)
+        _addToIndexRecursive(*i);
+}
 
 #define CREATE_OBJ(classname,str) \
-if(strcmp(classname::TYPENAME,str)==SAME)\
-{ if (create_with_root) nobj=new classname(this); else nobj=new classname();  return nobj; }
+if(strcmp(classname::TYPENAME,str)==SAME) \
+{ \
+  if (create_with_root) nobj=new classname(this); else nobj=new classname(); \
+  addToIndex(nobj); \
+  return nobj; \
+}
+
 
 FWObject *FWObjectDatabase::create(const string &type_name, bool create_with_root)
 {
     const char *n=type_name.c_str();
     FWObject   *nobj;
 
-    if(strcmp("AnyNetwork"  ,n)==SAME)  { nobj = new Network();   nobj->setXMLName("AnyNetwork");   return nobj; }
-    if(strcmp("AnyIPService",n)==SAME)  { nobj = new IPService(); nobj->setXMLName("AnyIPService"); return nobj; }
-    if(strcmp("AnyInterval" ,n)==SAME)  { nobj = new Interval();  nobj->setXMLName("AnyInterval");  return nobj; }
+    if (strcmp("comment",n)==SAME) return NULL;
+
+    if(strcmp("AnyNetwork"  ,n)==SAME)
+    {
+        nobj = new Network();
+        nobj->setXMLName("AnyNetwork");
+        addToIndex(nobj);
+        return nobj;
+    }
+
+    if(strcmp("AnyIPService",n)==SAME)
+    {
+        nobj = new IPService();
+        nobj->setXMLName("AnyIPService");
+        addToIndex(nobj);
+        return nobj;
+    }
+
+    if(strcmp("AnyInterval" ,n)==SAME)
+    {
+        nobj = new Interval();
+        nobj->setXMLName("AnyInterval");
+        addToIndex(nobj);
+        return nobj;
+    }
 
 
     CREATE_OBJ(Library             ,n);
     CREATE_OBJ(Policy              ,n);
     CREATE_OBJ(InterfacePolicy     ,n);
     CREATE_OBJ(NAT                 ,n);
+    CREATE_OBJ(Routing             ,n);
     CREATE_OBJ(PolicyRule          ,n);
     CREATE_OBJ(NATRule             ,n);
+    CREATE_OBJ(RoutingRule         ,n);
     CREATE_OBJ(RuleElementSrc      ,n);
     CREATE_OBJ(RuleElementDst      ,n);
     CREATE_OBJ(RuleElementSrv      ,n);
+    CREATE_OBJ(RuleElementItf      ,n);
     CREATE_OBJ(RuleElementOSrc     ,n);
     CREATE_OBJ(RuleElementODst     ,n);
     CREATE_OBJ(RuleElementOSrv     ,n);
@@ -321,10 +442,14 @@ FWObject *FWObjectDatabase::create(const string &type_name, bool create_with_roo
     CREATE_OBJ(RuleElementTDst     ,n);
     CREATE_OBJ(RuleElementTSrv     ,n);
     CREATE_OBJ(RuleElementInterval ,n);
+    CREATE_OBJ(RuleElementRDst     ,n);
+    CREATE_OBJ(RuleElementRGtw     ,n);
+    CREATE_OBJ(RuleElementRItf     ,n);
     CREATE_OBJ(HostOptions         ,n);
     CREATE_OBJ(FirewallOptions     ,n);
     CREATE_OBJ(PolicyRuleOptions   ,n);
     CREATE_OBJ(NATRuleOptions      ,n);
+    CREATE_OBJ(RoutingRuleOptions  ,n);
     CREATE_OBJ(Host                ,n);
     CREATE_OBJ(AddressRange        ,n);
     CREATE_OBJ(Management          ,n);
@@ -336,11 +461,14 @@ FWObject *FWObjectDatabase::create(const string &type_name, bool create_with_roo
     CREATE_OBJ(Interface           ,n);
     CREATE_OBJ(IPv4                ,n);
     CREATE_OBJ(physAddress         ,n);
+    CREATE_OBJ(DNSName             ,n);
+    CREATE_OBJ(AddressTable        ,n);
     CREATE_OBJ(FWObjectReference   ,n);
     CREATE_OBJ(FWServiceReference  ,n);
     CREATE_OBJ(FWIntervalReference ,n);
     CREATE_OBJ(TCPService          ,n);
     CREATE_OBJ(UDPService          ,n);
+    CREATE_OBJ(TagService          ,n);
     CREATE_OBJ(ICMPService         ,n);
     CREATE_OBJ(IPService           ,n);
     CREATE_OBJ(CustomService       ,n);
@@ -391,6 +519,11 @@ Firewall* FWObjectDatabase::findFirewallByName(const string &name) throw(FWExcep
  * library so we want to preserve subtree structure under obj. There
  * is no need to delete child objects, but we must remove all
  * references to them.
+ *
+ * Note: there is no need to search for references pointing at certain
+ * types of objects, such as references and rules/rule sets. This
+ * dramatically speeds up deleting firewalls with large policies and
+ * groups with lots of objects
  */
 void FWObjectDatabase::recursivelyRemoveObjFromTree(FWObject* obj,
                                                     bool remove_ref)
@@ -398,7 +531,11 @@ void FWObjectDatabase::recursivelyRemoveObjFromTree(FWObject* obj,
     obj->checkReadOnly();
 
     for (FWObject::iterator i=obj->begin(); i!=obj->end(); ++i)
+    {
+        if (FWReference::cast(*i)!=NULL ||
+            RuleSet::cast(*i)!=NULL) continue;
         recursivelyRemoveObjFromTree( *i , true);
+    }
 
     if (remove_ref)  removeAllReferences(obj);
     else             removeAllInstances(obj);
@@ -569,14 +706,14 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
          * find deleted objects library in src and check if any object
          * from it is present in dst
          */
-        FWObjectDatabase *srcroot = FWObjectDatabase::cast(src->getRoot());
-        FWObjectDatabase *dstroot = FWObjectDatabase::cast(dst->getRoot());
+        FWObjectDatabase *dstroot = dst->getRoot();
 
         /* commented out 11/12/04
          * If src has some deleted objects, we should just ignore them
          */
 
 #if 0
+        FWObjectDatabase *srcroot = src->getRoot();
         FWObject *srcdobj = srcroot->getById( dobjId );
         if (srcdobj)
         {
@@ -597,7 +734,7 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
          * it if they are present and not deleted in src
          */
         list<FWObject*> deletedObjects;
-        FWObject *dstdobj = dstroot->getById( dobjId );
+        FWObject *dstdobj = dstroot->findInIndex( dobjId );
         if (dstdobj)
         {
             for (FWObject::iterator i=dstdobj->begin(); i!=dstdobj->end(); i++)
