@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWOptions.cpp,v 1.3 2005/03/31 05:49:40 vkurland Exp $
+  $Id: FWOptions.cpp,v 1.5 2006/03/16 05:33:24 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -87,6 +87,9 @@ xmlNodePtr FWOptions::toXML(xmlNodePtr root) throw(FWException)
     {
         string name  = (*i).first;
         string value = (*i).second;
+
+        if (name[0]=='.') continue;
+        
         xmlChar *valbuf = xmlEncodeSpecialChars(NULL, STRTOXMLCAST(value) );
         opt=xmlNewChild(me,NULL,TOXMLCAST("Option"),valbuf);
         FREEXMLBUFF(valbuf);
@@ -112,4 +115,7 @@ const char *NATRuleOptions::TYPENAME={"NATRuleOptions"};
 NATRuleOptions::NATRuleOptions() : FWOptions() {}
 NATRuleOptions::NATRuleOptions(const FWObject *root) : FWOptions(root) {}
 
+const char *RoutingRuleOptions::TYPENAME={"RoutingRuleOptions"};
+RoutingRuleOptions::RoutingRuleOptions() : FWOptions() {}
+RoutingRuleOptions::RoutingRuleOptions(const FWObject *root) : FWOptions(root) {}
 
