@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWReference.cpp,v 1.2 2004/09/08 05:34:50 vkurland Exp $
+  $Id: FWReference.cpp,v 1.4 2006/03/05 04:15:42 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -97,11 +97,7 @@ void FWReference::setPointerId(const string &ref_id)
 
 FWObject *FWReference::getPointer()
 {
-    FWObject *ptr;
-
-    ptr=getRoot()->getById( getPointerId() , true );
-
-    return ptr;
+    return getRoot()->findInIndex( getPointerId() );
 }
 
 const string& FWReference::getPointerId()
