@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Firewall.cpp,v 1.3 2004/09/09 05:35:58 vkurland Exp $
+  $Id: Firewall.cpp,v 1.9 2006/04/19 15:03:21 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -24,6 +24,7 @@
 
 */
 
+#include <time.h>
 #include <assert.h>
 
 #include <fwbuilder/libfwbuilder-config.h>
@@ -40,6 +41,9 @@
 #include <fwbuilder/InterfacePolicy.h>
 #include <fwbuilder/NAT.h>
 
+#include <fwbuilder/Routing.h>
+#include <iostream>
+
 #include <fwbuilder/RuleElement.h>
 
 #include <fwbuilder/XMLTools.h>
@@ -55,16 +59,25 @@ Firewall::Firewall()
 {
     setStr("platform","unknown");
     setStr("host_OS" ,"unknown");
+    setInt("lastModified" ,0);
+    setInt("lastInstalled" ,0);
+    setInt("lastCompiled" ,0);
+    
 }
 
 Firewall::Firewall(const FWObject *root) : Host(root)
 {
     setStr("platform","unknown");
     setStr("host_OS" ,"unknown");
+    setInt("lastModified" ,0);
+    setInt("lastInstalled" ,0);
+    setInt("lastCompiled" ,0);
 
     add( new FirewallOptions() );
     add( new Policy() );
     add( new NAT() );
+    add( new Routing() );
+
 }
 
 
@@ -90,21 +103,50 @@ void Firewall::fromXML(xmlNodePtr root) throw(FWException)
         setStr("host_OS", n);
         FREEXMLBUFF(n);
     }
-
+    
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("lastModified")));
+    if(n!=NULL)
+    {
+        setStr("lastModified", n);
+        FREEXMLBUFF(n);
+    }
+    
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("lastInstalled")));
+    if(n!=NULL)
+    {
+        setStr("lastInstalled", n);
+        FREEXMLBUFF(n);
+    }
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("lastCompiled")));
+    if(n!=NULL)
+    {
+        setStr("lastCompiled", n);
+        FREEXMLBUFF(n);
+    }
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("inactive")));
+    if(n!=NULL)
+    {
+        setStr("inactive", n);
+        FREEXMLBUFF(n);
+    }
+    
     Host::fromXML(root);
+
 }
 
 xmlNodePtr Firewall::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
     FWObject *o;
-    
     o=getFirstByType( NAT::TYPENAME );
     if (o) o->toXML(me);
 
     o=getFirstByType( Policy::TYPENAME );
     if (o) o->toXML(me);
 
+    o=getFirstByType( Routing::TYPENAME );
+    if (o) o->toXML(me);
+
     for(FWObjectTypedChildIterator j=findByType(Interface::TYPENAME); j!=j.end(); ++j)
  	if((o=(*j))!=NULL )
  	    o->toXML(me);
@@ -135,6 +177,12 @@ NAT* Firewall::getNAT()
 }
 
 
+Routing* Firewall::getRouting()    
+{
+    return(Routing::cast(getFirstByType(Routing::TYPENAME)));
+}
+
+
 bool  Firewall::validateChild(FWObject *o)
 { 
     string otype=o->getTypeName();
@@ -143,6 +191,7 @@ bool  Firewall::validateChild(FWObject *o)
 	     otype==RuleSet::TYPENAME    ||
 	     otype==Policy::TYPENAME     ||
 	     otype==NAT::TYPENAME        ||
+	     otype==Routing::TYPENAME    ||
 	     otype==Management::TYPENAME ||
 	     otype==FirewallOptions::TYPENAME ));
 }
@@ -166,8 +215,6 @@ FWObject& Firewall::duplicate(const FWObject *obj, bool preserve_id) throw(FWExc
 {
     string err="Error creating object with type: ";
 
-//    return FWObject::duplicate(obj,preserve_id);
-
     checkReadOnly();
 
     shallowDuplicate(obj,preserve_id);
@@ -182,6 +229,10 @@ FWObject& Firewall::duplicate(const FWObject *obj, bool preserve_id) throw(FWExc
     FWObject *nat=addCopyOf(on,preserve_id);
     replaceRef(nat,obj->getId(), getId() );
 
+    Routing  *oroute =Routing::cast(obj->getFirstByType(Routing::TYPENAME));
+    FWObject *r=addCopyOf(oroute,preserve_id);
+    replaceRef(r,obj->getId(), getId() );  
+    
     FWObjectTypedChildIterator m=obj->findByType(Interface::TYPENAME);
     for ( ; m!=m.end(); ++m ) 
     {
@@ -240,3 +291,47 @@ FWObject& Firewall::duplicate(const FWObject *obj, bool preserve_id) throw(FWExc
     return *this;
 }
 
+void   Firewall::updateLastInstalledTimestamp()
+{
+    setInt("lastInstalled",time(NULL));
+}
+void Firewall::updateLastModifiedTimestamp()
+{
+    setInt("lastModified",time(NULL));
+}
+bool Firewall::needsInstall()
+{
+    if (getLastInstalled()==0 || getLastCompiled()==0) return true;
+    return !(getLastModified()<=getLastCompiled() && 
+            getLastCompiled()<=getLastInstalled());
+}
+bool Firewall::needsCompile()
+{
+    return getLastModified()>getLastCompiled() || getLastCompiled()==0;
+}
+time_t Firewall::getLastModified()
+{
+    return getInt("lastModified");
+    
+}
+time_t Firewall::getLastInstalled()
+{
+    return getInt("lastInstalled");
+}
+time_t Firewall::getLastCompiled()
+{
+    return getInt("lastCompiled");
+    
+}
+void   Firewall::updateLastCompiledTimestamp()
+{
+    setInt("lastCompiled",time(NULL));
+}
+bool   Firewall::getInactive()
+{
+    return getBool("inactive");
+}
+void   Firewall::setInactive(bool b)
+{
+    setBool("inactive",b);
+}
