@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Group.cpp,v 1.2 2004/09/08 05:34:50 vkurland Exp $
+  $Id: Group.cpp,v 1.3 2006/03/05 05:37:23 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -24,6 +24,8 @@
 
 */
 
+#include <iostream>
+
 #include <fwbuilder/libfwbuilder-config.h>
 
 #include <fwbuilder/FWObject.h>
@@ -32,6 +34,7 @@
 #include <fwbuilder/XMLTools.h>
 
 using namespace libfwbuilder;
+using namespace std;
 
 const char *Group::TYPENAME={"Group"};
 
@@ -39,7 +42,10 @@ const char *Group::TYPENAME={"Group"};
  *  Group should inherit a list of allowed types from its parent upon creation
  */
 Group::Group()  {}
-Group::Group(const FWObject *root) : FWObject(root) {}
+Group::Group(const FWObject *root) : FWObject(root)
+{
+    setRoot(root);   // do not remove!
+}
 
 Group::~Group() 
 {
