@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: IPAddress.h,v 1.3 2004/10/24 03:12:46 vkurland Exp $
+  $Id: IPAddress.h,v 1.4 2006/03/19 06:33:55 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -142,6 +142,12 @@ class Netmask: public IPAddress
 
 };
 
+class IPNetwork;
+std::vector<IPNetwork> getOverlap(const IPNetwork &n1,const IPNetwork &n2);
+std::vector<IPNetwork> substract(const IPNetwork &n1,const IPNetwork &n2);
+std::vector<IPNetwork> convertAddressRange(const IPAddress &start,
+                                           const IPAddress &end);
+
 class IPNetwork
 {
     private:
