@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Interface.cpp,v 1.2 2004/09/08 05:34:50 vkurland Exp $
+  $Id: Interface.cpp,v 1.4 2006/03/06 03:00:50 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -53,6 +53,7 @@ Interface::Interface():Address()
     setName("unknown");
     setBool("dyn",false);
     setBool("unnum",false);
+    setBool("bridgeport",false);
     setInt("security_level",0);
 
     bcast_bits       = 1    ;
@@ -65,6 +66,7 @@ Interface::Interface(const FWObject *root) : Address(root)
     setName("unknown");
     setBool("dyn",false);
     setBool("unnum",false);
+    setBool("bridgeport",false);
     setInt("security_level",0);
 
     bcast_bits       = 1    ;
@@ -130,6 +132,13 @@ void Interface::fromXML(xmlNodePtr root) throw(FWException)
         FREEXMLBUFF(n);
     }
 
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("bridgeport")));
+    if (n!=NULL)
+    {
+        setStr("bridgeport",n);
+        FREEXMLBUFF(n);
+    }
+
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("mgmt")));
     if (n!=NULL)
     {
@@ -204,6 +213,9 @@ bool Interface::isDyn() const { return(getBool("dyn")); }
 void Interface::setUnnumbered(bool value) { setBool("unnum",value); }
 bool Interface::isUnnumbered() const { return getBool("unnum"); }
 
+void Interface::setBridgePort(bool value) { setBool("bridgeport",value); }
+bool Interface::isBridgePort() const { return getBool("bridgeport"); }
+
 void Interface::setManagement(bool value) { setBool("mgmt",value); }
 bool Interface::isManagement() const { return (getBool("mgmt")); }
 
@@ -240,7 +252,7 @@ void  Interface::setPhysicalAddress(const std::string &paddr)
         pa->setPhysAddress(paddr);
     } else
     {
-        pa=physAddress::cast( FWObjectDatabase::db->create(physAddress::TYPENAME) );
+        pa=physAddress::cast( getRoot()->create(physAddress::TYPENAME) );
         pa->setPhysAddress(paddr);
         add(pa);
     }
@@ -286,7 +298,7 @@ Netmask   Interface::getNetmask() const
 
 IPv4*  Interface::addIPv4()
 {
-    IPv4* ipv4=IPv4::cast( FWObjectDatabase::db->create(IPv4::TYPENAME) );
+    IPv4* ipv4=IPv4::cast( getRoot()->create(IPv4::TYPENAME) );
     add(ipv4);
     return ipv4;
 }
