@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Interval.cpp,v 1.2 2004/09/08 05:34:50 vkurland Exp $
+  $Id: Interval.cpp,v 1.3 2006/03/05 04:15:42 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -206,13 +206,14 @@ void Interval::fromXML(xmlNodePtr root) throw(FWException)
 
 FWReference* Interval::createRef()
 {
-    FWIntervalReference *ref=new FWIntervalReference();
+//    FWIntervalReference *ref=new FWIntervalReference();
+    FWIntervalReference *ref=FWIntervalReference::cast(getRoot()->create(FWIntervalReference::TYPENAME, true));
     ref->setPointer(this);
     return ref;
 }
 
 bool Interval::isAny()
 { 
-    return getId()==FWObjectDatabase::cast(getRoot())->getAnyIntervalId();
+    return getId()==FWObjectDatabase::getAnyIntervalId();
 }
 
