@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: IntervalGroup.cpp,v 1.2 2004/09/08 05:34:50 vkurland Exp $
+  $Id: IntervalGroup.cpp,v 1.3 2006/03/05 04:15:42 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -28,6 +28,7 @@
 
 #include <fwbuilder/IntervalGroup.h>
 #include <fwbuilder/FWIntervalReference.h>
+#include <fwbuilder/FWObjectDatabase.h>
 
 using namespace std;
 using namespace libfwbuilder;
@@ -50,7 +51,8 @@ bool  IntervalGroup::validateChild(FWObject *o)
 
 FWReference* IntervalGroup::createRef()
 {
-    FWIntervalReference *ref=new FWIntervalReference();
+//    FWIntervalReference *ref=new FWIntervalReference();
+    FWIntervalReference *ref=FWIntervalReference::cast(getRoot()->create(FWIntervalReference::TYPENAME, true));
     ref->setPointer(this);
     return ref;
 }
