@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ObjectGroup.cpp,v 1.2 2004/09/08 05:34:51 vkurland Exp $
+  $Id: ObjectGroup.cpp,v 1.6 2006/03/23 05:50:01 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -31,6 +31,11 @@
 #include <fwbuilder/FWObjectReference.h>
 #include <fwbuilder/Address.h>
 #include <fwbuilder/AddressRange.h>
+#include <fwbuilder/DNSName.h>
+#include <fwbuilder/AddressTable.h>
+#include <fwbuilder/Service.h>
+#include <fwbuilder/ServiceGroup.h>
+#include <fwbuilder/FWServiceReference.h>
 
 using namespace std;
 using namespace libfwbuilder;
@@ -47,10 +52,9 @@ bool ObjectGroup::validateChild(FWObject *o)
     string otype=o->getTypeName();
 
     return (FWObject::validateChild(o) && 
-	    (Address::cast(o)!=NULL || 
-	     AddressRange::cast(o)!=NULL ||
-	     ObjectGroup::cast(o)!=NULL ||
-	     FWObjectReference::cast(o)!=NULL ));
+            Service::cast(o)==NULL &&
+            ServiceGroup::cast(o)==NULL &&
+            FWServiceReference::cast(o)==NULL);
 }
 
 
