@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: XMLTools.cpp,v 1.2 2004/09/08 05:34:52 vkurland Exp $
+  $Id: XMLTools.cpp,v 1.7 2006/05/15 04:06:08 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -215,41 +215,42 @@ void XMLTools::initXMLTools()
     xmlSetExternalEntityLoader(fwbExternalEntityLoader);
 }
 
-/*
- *  I use xmlParseMemory because libxml2 v2.6.2 broke xmlParseFile. If
- *  file name had a full path in it, it won't load and gave an error
- *  like this:
- *
- *  I/O warning : failed to load external entity "/resources.xml"
- *
- *  I do not check if xmlParseMemory succeeded in this method because
- *  I may need to throw different exceptions in different method using
- *  loadAndParseFile. loadAndParseFile just returns whatever
- *  xmlParseMemory returned, this works better in a sense that
- *  loadAndParseFile should be just a wrapper for xmlParseMemory and
- *  should replicate its behavior.
- *
- */
-xmlDocPtr XMLTools::loadAndParseFile(const std::string &rfile) throw(FWException)
+string XMLTools::readFile(const std::string &rfile) throw(FWException)
 {
     struct stat stt;
     int         fd;
 
+    if (rfile=="-") fd=0;
+    else
+    {
 #ifdef _WIN32
-    if (stat( rfile.c_str() , &stt )!=0 || (fd=_open(rfile.c_str(),O_RDONLY|_O_BINARY))<0)
+      if (stat( rfile.c_str() , &stt )!=0 || (fd=_open(rfile.c_str(),O_RDONLY|_O_BINARY))<0)
 #else
-    if (stat( rfile.c_str() , &stt )!=0 || (fd=open(rfile.c_str(),O_RDONLY))<0)
+      if (stat( rfile.c_str() , &stt )!=0 || (fd=open(rfile.c_str(),O_RDONLY))<0)
 #endif
         throw FWException("Could not read file "+rfile);
 
-    char *buf=(char*)malloc(stt.st_size);
-    if (!buf) throw FWException("Out of memory");
+    }
 
+    string buf;
+    int chunk_size = 65536;
+    char *chunk = (char*)malloc(chunk_size);
+    if (!chunk) throw FWException("Out of memory");
+
+    int   n = 0;
+    while(1)
+    {
 #ifdef _WIN32
-    int n=_read(fd,buf,stt.st_size);
+        n=_read(fd,chunk,chunk_size-1);
 #else
-    int n=read(fd,buf,stt.st_size);
+        n=read(fd,chunk,chunk_size-1);
 #endif
+        if (n<=0) break;
+        chunk[n] = '\0';
+
+        buf = buf + chunk;
+    }
+    free(chunk);
 
     int errn=errno;
 #ifdef _WIN32
@@ -261,19 +262,16 @@ xmlDocPtr XMLTools::loadAndParseFile(const std::string &rfile) throw(FWException
     if (n<0)
     {
         string s;
-        s = "Error reading file " + rfile + " : " + string(strerror(errn));
+        s = "Error reading from file " + rfile + " : " + string(strerror(errn));
         throw FWException(s);
     }
-
-    xmlDocPtr doc = xmlParseMemory(buf, stt.st_size);
-    memset(buf,0,stt.st_size);
-    free(buf);
-
-    return doc;
+    return buf;
 }
 
 xmlDocPtr XMLTools::parseFile(const string &file_name, 
-                              bool use_dtd, const string &template_dir) throw(FWException)
+                              const string &buffer,
+                              bool use_dtd,
+                              const string &template_dir) throw(FWException)
 {
     xml_parser_mutex.lock();
 
@@ -286,7 +284,9 @@ xmlDocPtr XMLTools::parseFile(const string &file_name,
     string errors;
     xmlSetGenericErrorFunc (&errors, xslt_error_handler);
 //    xmlDocPtr doc = xmlParseFile(file_name.c_str()); 
-    xmlDocPtr doc = loadAndParseFile(file_name);
+
+    xmlDocPtr doc = xmlParseMemory(buffer.c_str(), buffer.length());
+
     xmlSetGenericErrorFunc (NULL, NULL);
 
     xml_parser_mutex.unlock();
@@ -317,11 +317,13 @@ xmlDocPtr XMLTools::loadFile(const string &data_file ,
          << endl;
 #endif
     
-    if(access(data_file.c_str() , R_OK )!=0)
+    if(data_file!="-" && access(data_file.c_str() , R_OK )!=0)
         throw FWException(string("Could not access data file: ")+data_file);
-    
+
+    string buf = readFile(data_file);
+
     // First load without using DTD to check version
-    xmlDocPtr doc = parseFile(data_file, false, template_dir); 
+    xmlDocPtr doc = parseFile(data_file, buf, false, template_dir); 
 
 #ifdef FW_XMLTOOLS_VERBOSE
     cerr << "Parsed file: " << data_file << endl;
@@ -379,7 +381,7 @@ in the same directory with extension '.bak'. Are you sure you want to open it?";
     
     // Now we know the version is OK,
     // let us load for real, checking DTD.
-    doc = parseFile(data_file, true, template_dir); 
+    doc = parseFile(data_file, readFile(data_file), true, template_dir); 
     
     return doc;
 }
@@ -460,6 +462,22 @@ void XMLTools::saveFile(xmlDocPtr doc,
 }
 
 
+void XMLTools::dumpToMemory(xmlDocPtr doc, 
+                            xmlChar **buffer,
+                            int      *size,
+                            const string &type_name,
+                            const string &dtd_file) throw(FWException)
+{
+    setDTD(doc, type_name, dtd_file);
+
+#ifdef HAVE_XMLSAVEFORMATFILEENC
+    xmlDocDumpFormatMemoryEnc(doc,buffer,size,"utf-8",1);
+#else
+    xmlDocDumpMemoryEnc(doc,buffer,size,"utf-8");
+#endif
+}
+
+
 void XMLTools::transformFileToFile(const string &src_file,
 				   const string &stylesheet_file,
 				   const char **params,
@@ -500,9 +518,8 @@ void XMLTools::transformFileToFile(const string &src_file,
         );
     }
     
-
-//    doc = xmlParseFile(src_file.c_str());
-    doc = loadAndParseFile(src_file);
+    string buffer = readFile(src_file);
+    doc = xmlParseMemory(buffer.c_str(), buffer.length());
 
     res = xsltApplyStylesheet(ss, doc, params);
     xsltSaveResultToFilename(dst_file.c_str(), res, ss, 0);
