@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland <vadim@vk.crocodile.org>
 
-  $Id: physAddress.cpp,v 1.2 2004/09/08 05:34:52 vkurland Exp $
+  $Id: physAddress.cpp,v 1.3 2006/03/05 04:15:42 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -85,7 +85,8 @@ void physAddress::setPhysAddress(const std::string &s)
 
 FWReference* physAddress::createRef()
 {
-    FWObjectReference *ref=new FWObjectReference();
+//    FWObjectReference *ref=new FWObjectReference();
+    FWObjectReference *ref=FWObjectReference::cast(getRoot()->create(FWObjectReference::TYPENAME, true));
     ref->setPointer(this);
     return ref;
 }
