@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Compiler.cpp,v 1.5 2004/12/23 07:19:16 vkurland Exp $
+  $Id: Compiler.cpp,v 1.25 2006/05/21 04:42:50 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -42,6 +42,8 @@
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/InterfacePolicy.h"
+#include "fwbuilder/DNSName.h"
+#include "fwbuilder/MultiAddress.h"
 
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/XMLTools.h"
@@ -52,6 +54,9 @@
 #include <iomanip>
 #include <algorithm>
 #include <functional>
+#include <fstream>
+#include <string>
+
  
 using namespace libfwbuilder;
 using namespace fwcompiler;
@@ -90,10 +95,12 @@ int Compiler::prolog()
 
 /* caching firewall interfaces */
     FWObjectTypedChildIterator j=fw->findByType(Interface::TYPENAME);
-    for ( ; j!=j.end(); ++j ) {
+    for ( ; j!=j.end(); ++j )
+    {
         Interface *interface_=Interface::cast(*j);
         fw_interfaces[interface_->getId()]=interface_;
     }
+    
     fw_id=fw->getId();
 
     fwopt = fw->getOptionsObject();
@@ -111,6 +118,11 @@ void Compiler::epilog()
     exit(1);
 }
 
+int    Compiler::getCompiledScriptLength()
+{
+    return int(output.tellp());
+}
+
 string Compiler::getCompiledScript() 
 { 
     string res;
@@ -154,7 +166,10 @@ void Compiler::_init(FWObjectDatabase *_db, const string &fwobjectname)
 Compiler::Compiler(FWObjectDatabase *_db, 
 		   const string &fwobjectname)
 {
-    test_mode=false;
+    source_ruleset = NULL;
+    ruleSetName = "";
+    test_mode = false;
+    osconfigurator = NULL;
     _init(_db,fwobjectname);
 }
 
@@ -162,9 +177,28 @@ Compiler::Compiler(FWObjectDatabase *_db,
 		   const string &fwobjectname,
 		   OSConfigurator *_oscnf)
 {
-    test_mode=false;
+    source_ruleset = NULL;
+    ruleSetName = "";
+    test_mode = false;
+    osconfigurator = _oscnf;
     _init(_db,fwobjectname);
-    osconfigurator=_oscnf;
+}
+
+// this constructor is used by class Preprocessor, it does not call _init
+Compiler::Compiler(FWObjectDatabase *_db)
+{
+    source_ruleset = NULL;
+    ruleSetName = "";
+    test_mode = false;
+    osconfigurator = NULL;
+    initialized = false;
+    _cntr_ = 1; 
+    fw = NULL; 
+    temp_ruleset = NULL; 
+    combined_ruleset = NULL;
+    debug = 0;
+    debug_rule = -1;
+    verbose = true;
 }
 
 string Compiler::createRuleLabel(const string &txt,
@@ -238,10 +272,19 @@ void Compiler::compile()
 
 void Compiler::_expand_group_recursive(FWObject *o,list<FWObject*> &ol)
 {
-    if (Group::cast( o )!=NULL) {
+/* special case: MultiAddress. This class inherits ObjectGroup, but
+ * should not be expanded if it is expanded at run time
+ *
+ * This is now redundant since we use class MultiAddressRunTime for
+ * run-time address tables
+ */
+    MultiAddress *adt = MultiAddress::cast(o);
+    if ((Group::cast(o)!=NULL && adt==NULL) ||
+        (adt!=NULL && adt->isCompileTime()))
+    {
 	for (FWObject::iterator i2=o->begin(); i2!=o->end(); ++i2) {
 	    FWObject *o1= *i2;
-	    if (FWReference::cast(o1)!=NULL) o1=getCachedObj(o1->getStr("ref"));
+	    if (FWReference::cast(o1)!=NULL) o1=FWReference::cast(o1)->getPointer();
 	    assert(o1);
 
 	    _expand_group_recursive(o1,ol);
@@ -261,7 +304,7 @@ void Compiler::expandGroupsInRuleElement(RuleElement *s)
     list<FWObject*> cl;
     for (FWObject::iterator i1=s->begin(); i1!=s->end(); ++i1) {
         FWObject *o= *i1;
-        if (FWReference::cast(o)!=NULL) o=getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         assert(o);
         _expand_group_recursive(o,cl);
     }
@@ -279,19 +322,21 @@ void Compiler::_expand_addr_recursive(Rule *rule,FWObject *s,list<FWObject*> &ol
     Interface *rule_iface = fw_interfaces[rule->getInterfaceId()];
     bool on_loopback= ( rule_iface && rule_iface->isLoopback() );
 
-//    cerr << " object s: " << s->getName() << " " << s->getTypeName() << endl;
+    //cerr << "Rule " << rule->getLabel() << " object s: " << s->getName() << " " << s->getTypeName() << endl;
 
     list<FWObject*> addrlist;
 
     for (FWObject::iterator i1=s->begin(); i1!=s->end(); ++i1) 
     {
 	FWObject *o= *i1;
-	if (FWReference::cast(o)!=NULL) o=getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	assert(o);
 
-        if (Address::cast(o)!=NULL) addrlist.push_back(o);
+        if (Address::cast(o)!=NULL || MultiAddress::cast(o)!=NULL) addrlist.push_back(o);
     }
 
+    //cerr << "Rule " << rule->getLabel() << "  addrlist.size()=" << addrlist.size() << endl;
+
     if (addrlist.empty()) ol.push_back(s);
     else
     {
@@ -318,9 +363,9 @@ void Compiler::_expand_addr_recursive(Rule *rule,FWObject *s,list<FWObject*> &ol
 void Compiler::_expandInterface(Interface *iface,  std::list<FWObject*> &ol)
 {
 /*
- * if this is unnumbered interface, then do not use it
+ * if this is unnumbered interface or a bridge port, then do not use it
  */
-    if (iface->isUnnumbered()) return;
+    if (iface->isUnnumbered() || iface->isBridgePort()) return;
 
 /*
  * if this is an interface with dynamic address, then simply use it
@@ -376,10 +421,17 @@ void Compiler::_expandAddr(Rule *rule,FWObject *s)
 
     _expand_addr_recursive(rule,s,cl);
 
-    if ( ! cl.empty() )     {
+    if ( ! cl.empty() )
+    {
 	s->clearChildren();
-
-	for (FWObject::iterator i1=cl.begin(); i1!=cl.end(); ++i1)  {
+	for (FWObject::iterator i1=cl.begin(); i1!=cl.end(); ++i1) 
+        {
+/*
+            cerr << "Compiler::_expandAddr: adding object: (*i1)->name=";
+            cerr << (*i1)->getName();
+            cerr << "  (*i1)->id=" << (*i1)->getId();
+            cerr << endl;
+*/
 	    s->addRef( *i1 );
 	}
     }
@@ -391,7 +443,7 @@ void Compiler::_expandAddressRanges(Rule *rule,FWObject *s)
     for (FWObject::iterator i1=s->begin(); i1!=s->end(); ++i1) 
     {
 	FWObject *o= *i1;
-	if (FWReference::cast(o)!=NULL) o=getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	assert(o!=NULL);
 
 	if (AddressRange::cast(o)==NULL) cl.push_back(o);
@@ -448,7 +500,7 @@ bool Compiler::_complexMatchWithInterface(Address   *obj1,
             obj1_pa->getPhysAddress()==iface_pa->getPhysAddress()) return true;
     }
 
-    if ( ! iface->isDyn() && ! iface->isUnnumbered() ) 
+    if ( iface->isRegular() )
     {
         FWObjectTypedChildIterator k=iface->findByType(IPv4::TYPENAME);
         for ( ; k!=k.end(); ++k ) 
@@ -547,8 +599,7 @@ bool Compiler::complexMatch(Address *obj1,
           (recognize_multicasts && obj1_addr.isMulticast()) )
     ) return true;
 
-    if (Interface::cast(obj1)!=NULL && 
-        (Interface::cast(obj1)->isDyn() || Interface::cast(obj1)->isUnnumbered()))
+    if (Interface::cast(obj1)!=NULL && !Interface::cast(obj1)->isRegular())
         return false;
 
     if (Interface::cast(obj2)!=NULL)
@@ -581,7 +632,7 @@ Interface* Compiler::findInterfaceFor(const Address *obj1, const Address *obj2)
 
         if (iface->getId()      == obj1->getId() )      return iface;
 
-        if ( ! iface->isDyn() && ! iface->isUnnumbered() ) 
+        if ( iface->isRegular() )
         {
             FWObjectTypedChildIterator k=iface->findByType(IPv4::TYPENAME);
             for ( ; k!=k.end(); ++k ) 
@@ -617,7 +668,7 @@ Address* Compiler::findAddressFor(const Address *obj1,const Address *obj2)
 
         if (iface->getId()      == obj1->getId() )      return iface;
 
-        if ( ! iface->isDyn() && ! iface->isUnnumbered() ) 
+        if ( iface->isRegular() )
         {
             FWObjectTypedChildIterator k=iface->findByType(IPv4::TYPENAME);
             for ( ; k!=k.end(); ++k ) 
@@ -701,6 +752,11 @@ void Compiler::deleteRuleProcessors()
     rule_processors.clear();
 }
 
+Compiler::Begin::Begin() : BasicRuleProcessor("")
+{
+    init=false;
+};
+
 Compiler::Begin::Begin(const std::string &n) : BasicRuleProcessor(n) 
 {
     init=false;
@@ -709,9 +765,11 @@ Compiler::Begin::Begin(const std::string &n) : BasicRuleProcessor(n)
 bool Compiler::Begin::processNext()
 {
     assert(compiler!=NULL);
-    if (!init) {
+    if (!init)
+    {
         for (FWObject::iterator i=compiler->combined_ruleset->begin();
-             i!=compiler->combined_ruleset->end(); ++i) {
+             i!=compiler->combined_ruleset->end(); ++i)
+        {
             Rule *rule=Rule::cast(*i);
 
             Rule  *r= Rule::cast(compiler->dbcopy->create(rule->getTypeName(),true) );
@@ -721,7 +779,7 @@ bool Compiler::Begin::processNext()
             tmp_queue.push_back( r );
         }
         init=true;
-        cout << " " << name << endl << flush;
+        if (!name.empty()) cout << " " << name << endl << flush;
         return true;
     }
     return false;
@@ -827,7 +885,7 @@ bool Compiler::splitIfRuleElementMatchesFW::processNext()
 
 	FWObject *o   = *i1;
 	FWObject *obj = NULL;
-	if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
         Address *a=Address::cast(obj);
         assert(a!=NULL);
 
@@ -878,7 +936,7 @@ bool Compiler::eliminateDuplicatesInRE::processNext()
     for(list<FWObject*>::iterator i=re->begin(); i!=re->end(); ++i) {
         FWObject *o= *i;
 	FWObject *obj = NULL;
-        if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
 
         comparator->set(obj);
 
@@ -911,7 +969,7 @@ void  Compiler::recursiveGroupsInRE::isRecursiveGroup(const string &grid,FWObjec
     for (FWObject::iterator i=obj->begin(); i!=obj->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         if (Group::cast(o)!=NULL)
         {
             if (o->getId()==grid)
@@ -939,7 +997,7 @@ bool Compiler::recursiveGroupsInRE::processNext()
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         if (Group::cast(o)!=NULL)  isRecursiveGroup(o->getId(),o);
     }
 
@@ -959,7 +1017,7 @@ int  Compiler::emptyGroupsInRE::countChildren(FWObject *obj)
     for (FWObject::iterator i=obj->begin(); i!=obj->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         if (Group::cast(o)!=NULL) n+=countChildren(o); // group itself does not count since it can be empty, too
         else n++;   // but if it is not a group, then we count it.
     }
@@ -981,8 +1039,8 @@ bool Compiler::emptyGroupsInRE::processNext()
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
-        if (Group::cast(o)!=NULL && countChildren(o)==0)
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        if (Group::cast(o)!=NULL && MultiAddress::cast(o)==NULL && countChildren(o)==0)
             cl.push_back(o);
     }
 
@@ -1032,6 +1090,61 @@ bool Compiler::emptyGroupsInRE::processNext()
     return true;
 }
 
+
+/**
+ * swaps MultiAddress objects that require run-time expansion with
+ * MultiAddressRunTime equivalents
+ */
+bool Compiler::swapMultiAddressObjectsInRE::processNext()
+{
+    Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
+
+    RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
+
+    list<MultiAddress*> cl;
+    for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        if (MultiAddress::cast(o)!=NULL && MultiAddress::cast(o)->isRunTime())
+            cl.push_back(MultiAddress::cast(o));
+    }
+
+    if (!cl.empty())
+    {
+        for (list<MultiAddress*>::iterator i=cl.begin(); i!=cl.end(); i++)
+        {
+            MultiAddress *ma = *i;
+
+            string mart_id = ma->getId()+"_runtime";
+            MultiAddressRunTime *mart = 
+                MultiAddressRunTime::cast(compiler->dbcopy->findInIndex(mart_id));
+            if (mart==NULL)
+            {
+                mart = new MultiAddressRunTime(ma);
+
+                // need to ensure stable ID for the runtime object, so
+                // that when the same object is replaced in different
+                // rulesets by different compiler passes, chosen
+                // runtime object has the same ID and is identified as
+                // the same by the compiler.
+
+                mart->setId( mart_id );
+                compiler->dbcopy->addToIndex(mart);
+                compiler->dbcopy->add(mart);
+            }
+            re->removeRef(ma);
+            re->addRef(mart);
+        }
+        tmp_queue.push_back(rule);
+        return true;
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+
 bool Compiler::catchUnnumberedIfaceInRE(RuleElement *re)
 {
     bool err=false;
@@ -1039,9 +1152,16 @@ bool Compiler::catchUnnumberedIfaceInRE(RuleElement *re)
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=getCachedObj(o->getStr("ref"));
-	assert(o!=NULL);
-        err |= ((iface=Interface::cast(o))!=NULL && iface->isUnnumbered() );
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        if (o==NULL)
+        {
+            Rule *rule = Rule::cast(re->getParent());
+            string errmsg = "catchUnnumberedIfaceInRE: Can't find object in cache, ID=" + (*i)->getStr("ref") + "  rule " + rule->getLabel();
+            abort(errmsg);
+        }
+        err |= ((iface=Interface::cast(o))!=NULL &&
+                (iface->isUnnumbered() || iface->isBridgePort())
+        );
     }
     return err;
 }
@@ -1053,7 +1173,7 @@ Address* Compiler::getFirstSrc(const PolicyRule *rule)
 
     FWObject *o=src->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Address::cast(o);
 }
@@ -1064,7 +1184,7 @@ Address* Compiler::getFirstDst(const PolicyRule *rule)
 
     FWObject *o=dst->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Address::cast(o);
 }
@@ -1075,7 +1195,7 @@ Service* Compiler::getFirstSrv(const PolicyRule *rule)
 
     FWObject *o=srv->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Service::cast(o);
 }
@@ -1087,18 +1207,30 @@ Interval* Compiler::getFirstWhen(const PolicyRule *rule)
 
     FWObject *o=when->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Interval::cast(o);
 }
 
+Interface* Compiler::getFirstItf(const PolicyRule *rule)
+{
+    RuleElementItf *itf=rule->getItf();
+    if (itf==NULL) return NULL;  // itf is optional element
+
+    FWObject *o=itf->front();
+    if (o && FWReference::cast(o)!=NULL)
+        o=FWReference::cast(o)->getPointer();
+
+    return Interface::cast(o);
+}
+
 Address* Compiler::getFirstOSrc(const NATRule *rule)
 {
     RuleElementOSrc *osrc=rule->getOSrc();
 
     FWObject *o=osrc->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Address::cast(o);
 }
@@ -1109,7 +1241,7 @@ Address* Compiler::getFirstODst(const NATRule *rule)
 
     FWObject *o=odst->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Address::cast(o);
 }
@@ -1120,7 +1252,7 @@ Service* Compiler::getFirstOSrv(const NATRule *rule)
 
     FWObject *o=osrv->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Service::cast(o);
 }
@@ -1133,7 +1265,7 @@ Address* Compiler::getFirstTSrc(const NATRule *rule)
 
     FWObject *o=tsrc->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Address::cast(o);
 }
@@ -1144,7 +1276,7 @@ Address* Compiler::getFirstTDst(const NATRule *rule)
 
     FWObject *o=tdst->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Address::cast(o);
 }
@@ -1155,7 +1287,7 @@ Service* Compiler::getFirstTSrv(const NATRule *rule)
 
     FWObject *o=tsrv->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     return Service::cast(o);
 }
