@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Compiler.h,v 1.2 2004/09/08 05:34:53 vkurland Exp $
+  $Id: Compiler.h,v 1.13 2006/05/16 22:39:36 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -122,7 +122,7 @@ namespace fwcompiler {
     class Compiler {
 
         void _init(libfwbuilder::FWObjectDatabase *_db,
-		   const std::string &fwname);
+                   const std::string &fwname);
 
         virtual void _expand_group_recursive(libfwbuilder::FWObject *o,
 				     std::list<libfwbuilder::FWObject*> &ol);
@@ -210,10 +210,11 @@ namespace fwcompiler {
          * reference. Uses cache, therefore is faster than
          * RuleElement::getFirst(true)
          */
-        libfwbuilder::Address* getFirstSrc(const libfwbuilder::PolicyRule *rule);
-        libfwbuilder::Address* getFirstDst(const libfwbuilder::PolicyRule *rule);
-        libfwbuilder::Service* getFirstSrv(const libfwbuilder::PolicyRule *rule);
-        libfwbuilder::Interval* getFirstWhen(const libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Address*   getFirstSrc(const libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Address*   getFirstDst(const libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Service*   getFirstSrv(const libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Interval*  getFirstWhen(const libfwbuilder::PolicyRule *rule);
+        libfwbuilder::Interface* getFirstItf(const libfwbuilder::PolicyRule *rule);
         
         libfwbuilder::Address* getFirstOSrc(const libfwbuilder::NATRule *rule);
         libfwbuilder::Address* getFirstODst(const libfwbuilder::NATRule *rule);
@@ -238,6 +239,7 @@ namespace fwcompiler {
         {
             bool init;
             public:
+            Begin();
             Begin(const std::string &n);
             virtual bool processNext();
         };
@@ -365,6 +367,18 @@ namespace fwcompiler {
         };
 
 
+        /**
+         * Replace MultiAddress objects that require run-time address
+         * expansion with corresponding MultiAddressRunTime objects
+         */
+        class swapMultiAddressObjectsInRE : public BasicRuleProcessor
+        {
+            std::string re_type;
+            public:
+            swapMultiAddressObjectsInRE(const std::string &name,
+                      const std::string &t) : BasicRuleProcessor(name) { re_type=t; }
+            virtual bool processNext();
+        };
 
 	/**
 	 * generic rule debugger: prints name of the previous rule
@@ -521,8 +535,12 @@ namespace fwcompiler {
 	libfwbuilder::FWObjectDatabase    *dbcopy;
 	libfwbuilder::Firewall            *fw;
 
+        std::string                        ruleSetName;;
+        
+	libfwbuilder::RuleSet             *source_ruleset;
 	libfwbuilder::RuleSet             *combined_ruleset;
 	libfwbuilder::RuleSet             *temp_ruleset;
+
         libfwbuilder::Group               *temp;
 
 	std::stringstream                  output;
@@ -538,13 +556,18 @@ namespace fwcompiler {
 		 const std::string &fwname,
 		 fwcompiler::OSConfigurator *_oscnf);
 
+	Compiler(libfwbuilder::FWObjectDatabase *_db);
+        
 	void setDebugLevel(int dl) { debug=dl;       }
 	void setDebugRule(int dr)  { debug_rule=dr;  }
 	void setVerbose(bool v)    { verbose=v;      }
         void setTestMode()         { test_mode=true; }
-
+        void setSourceRuleSet(libfwbuilder::RuleSet *rs) { source_ruleset = rs; }
+        void setRuleSetName(const std::string &name) { ruleSetName = name; }
+        std::string getRuleSetName() { return ruleSetName; }
+        
 	std::string getCompiledScript();
-
+        int         getCompiledScriptLength();
 
 	void expandGroupsInRuleElement(libfwbuilder::RuleElement *s);
 
