@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler.cpp,v 1.2 2004/09/08 05:34:53 vkurland Exp $
+  $Id: NATCompiler.cpp,v 1.5 2006/05/16 22:39:36 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -71,10 +71,17 @@ int NATCompiler::prolog()
  */
     int global_num=0;
 
-	
-    list<FWObject*> l3=nat->getByType(NATRule::TYPENAME);
-    for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) {
-	Rule *r= Rule::cast(*j);
+
+//    list<FWObject*> l3=nat->getByType(NATRule::TYPENAME);
+//    for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) {
+
+    FWObject *ruleset = source_ruleset;
+    if (ruleset == NULL) ruleset = nat;
+
+    for (FWObject::iterator i=ruleset->begin(); i!=ruleset->end(); i++)
+    {
+
+	Rule *r= Rule::cast(*i);
 	if (r->isDisabled()) continue;
 	r->setInterfaceId("");
 	r->setLabel( createRuleLabel("NAT", r->getPosition()) );
@@ -134,8 +141,8 @@ bool NATCompiler::classifyNATRule::processNext()
 
     RuleElementTDst *tdstre=rule->getTDst();
 
-    Address  *osrc=compiler->getFirstOSrc(rule);
-    Address  *odst=compiler->getFirstODst(rule);
+//    Address  *osrc=compiler->getFirstOSrc(rule);
+//    Address  *odst=compiler->getFirstODst(rule);
     Service  *osrv=compiler->getFirstOSrv(rule);
 
     Address  *tsrc=compiler->getFirstTSrc(rule);
@@ -563,7 +570,7 @@ bool NATCompiler::MACFiltering::checkRuleElement(RuleElement *re)
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
         FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         if (physAddress::isA(o)) 
         {
             lst.push_back(o);
@@ -628,7 +635,7 @@ bool NATCompiler::splitODstForSNAT::processNext()
             for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++)
             {
                 FWObject *o= *i;
-                if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+                if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
                 Address *a=Address::cast(o);
                 string iid="";
                 Interface *iface=compiler->findInterfaceFor( a , compiler->fw );
