@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler.cpp,v 1.5 2004/11/11 07:00:18 vkurland Exp $
+  $Id: PolicyCompiler.cpp,v 1.13 2006/06/15 05:05:02 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -49,6 +49,7 @@
 #include "fwbuilder/XMLTools.h"
 #include "fwbuilder/FWException.h"
 #include "fwbuilder/Group.h"
+#include "fwbuilder/MultiAddress.h"
 
 #include <iostream>
 #include <iomanip>
@@ -72,6 +73,9 @@ int PolicyCompiler::prolog()
     temp_ruleset = new Policy();   // working copy of the policy
     fw->add( temp_ruleset );
 
+    int global_num=0;
+
+#ifdef USE_INTERFACE_POLICY
 
 /*
  *  build combined policy by collapsing all the rules together.
@@ -80,8 +84,6 @@ int PolicyCompiler::prolog()
  *  also calculate global numbers for all rules and store them, too.
  *  These are used to detect rule shadowing.
  */
-    int global_num=0;
-
     list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
     for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) 
     {
@@ -104,22 +106,42 @@ int PolicyCompiler::prolog()
             }
 	}
     }
+#endif
 
-	
-    list<FWObject*> l3=policy->getByType(PolicyRule::TYPENAME);
-    for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) 
+    FWObject *ruleset = source_ruleset;
+    if (ruleset == NULL) ruleset = policy;
+
+    for (FWObject::iterator i=ruleset->begin(); i!=ruleset->end(); i++)
     {
-	Rule *r= Rule::cast(*j);
+	PolicyRule *r= PolicyRule::cast(*i);
 	if (r->isDisabled()) continue;
-	r->setInterfaceId("");
-	r->setLabel( createRuleLabel("global", r->getPosition()) );
+
+        RuleElementItf *itfre=r->getItf();   assert(itfre);
+
+        if (itfre->isAny())
+        {
+//          r->setInterfaceId("");
+            r->setLabel( createRuleLabel("global", r->getPosition()) );
+        } else
+        {
+//          r->setInterfaceId( itf->getId() );
+            string interfaces = "";
+            for (FWObject::iterator i=itfre->begin(); i!=itfre->end(); ++i)
+            {
+                FWObject *o=*i;
+                if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+                if (interfaces!="") interfaces += ",";
+                interfaces += o->getName();
+            }
+            r->setLabel( createRuleLabel(interfaces, r->getPosition()) );
+        }
 	r->setAbsRuleNumber(global_num); global_num++;
         r->setUniqueId( r->getId() );
 	combined_ruleset->add( r );
     }
 
     initialized=true;
-    
+
     return combined_ruleset->size();
 }
 
@@ -140,6 +162,16 @@ bool PolicyCompiler::checkForShadowing(const PolicyRule &r1,const PolicyRule &r2
     if (r1.getAction()==PolicyRule::Accounting  ||
         r2.getAction()==PolicyRule::Accounting ) return false;
 
+    /*
+     * TODO: actually, route rule may shadow other rules if it
+     * translates into "final" target, that is stops processing. This
+     * may or may not be so, depending on the platform and combination
+     * of rule options.
+     */
+    if (r1.getAction()==PolicyRule::Route  ||
+        r2.getAction()==PolicyRule::Route ) return false;
+
+
     Address  *src1=getFirstSrc(&r1);
     Address  *dst1=getFirstDst(&r1);
     Service  *srv1=getFirstSrv(&r1);
@@ -220,6 +252,69 @@ bool PolicyCompiler::cmpRules(const PolicyRule &r1,
     return ( (*src1 == *src2) && (*dst1 == *dst2) && (*srv1 == *srv2) );
 }
 
+bool PolicyCompiler::ItfNegation::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+//    FWOptions  *ruleopt =rule->getOptionsObject();
+
+    list<FWObject*> allInterfaces=compiler->fw->getByType(Interface::TYPENAME);
+
+    RuleElementItf *itfre=rule->getItf();   assert(itfre);
+    if (itfre->getNeg())
+    {
+        for (FWObject::iterator i=itfre->begin(); i!=itfre->end(); ++i)
+        {
+            FWObject *o=*i;
+            if (FWReference::cast(o)!=NULL)
+                o=FWReference::cast(o)->getPointer();
+
+            allInterfaces.remove(o);
+        }
+        itfre->reset();
+        itfre->setNeg(false);
+        for (FWObject::iterator i=allInterfaces.begin(); i!=allInterfaces.end(); ++i)
+            itfre->addRef(*i);
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+bool  PolicyCompiler::InterfacePolicyRules::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+
+    RuleElementItf *itfre=rule->getItf();   assert(itfre);
+    if (itfre->isAny())
+    {
+        rule->setInterfaceId("");
+        tmp_queue.push_back(rule);
+        return true;
+    }
+    if (itfre->size()==1)
+    {
+        Interface *itf = compiler->getFirstItf(rule);  assert(itf);
+        rule->setInterfaceId( itf->getId() );
+        tmp_queue.push_back(rule);
+        return true;
+    } else
+    {
+        for (FWObject::iterator i=itfre->begin(); i!=itfre->end(); ++i)
+        {
+            FWObject *o=*i;
+            if (FWReference::cast(o)!=NULL)
+                o=FWReference::cast(o)->getPointer();
+            PolicyRule *r= PolicyRule::cast(
+                compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+            r->duplicate(rule);
+            r->setInterfaceId(o->getId());
+            compiler->temp_ruleset->add(r);
+            tmp_queue.push_back(r);
+        }
+    }
+    return true;
+}
+
 
 bool  PolicyCompiler::ExpandGroups::processNext()
 {
@@ -404,7 +499,7 @@ bool PolicyCompiler::splitServices::processNext()
     for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++)
     {
         FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
         Service *s=Service::cast( o );
         assert(s);
@@ -457,7 +552,7 @@ bool PolicyCompiler::separateTCPWithFlags::processNext()
     for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++)
     {
         FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    
         TCPService *s=TCPService::cast( o );
         if (s==NULL) continue;
@@ -494,7 +589,7 @@ bool PolicyCompiler::verifyCustomServices::processNext()
 
     for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++) {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	assert(o!=NULL);
 	if (CustomService::isA(o) && 
 	    CustomService::cast(o)->getCodeForPlatform(compiler->myPlatformName()).empty())
@@ -519,12 +614,18 @@ Address* PolicyCompiler::checkForZeroAddr::findZeroAddress(RuleElement *re)
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	assert(o!=NULL);
+
+        MultiAddress *maddr = MultiAddress::cast(o);
+        if (maddr && maddr->isRunTime()) continue;
+
         Address *addr=Address::cast(o);
 
         if (Interface::cast(o)!=NULL && 
-            (Interface::cast(o)->isDyn() || Interface::cast(o)->isUnnumbered()))
+            (Interface::cast(o)->isDyn() ||
+             Interface::cast(o)->isUnnumbered() ||
+             Interface::cast(o)->isBridgePort()))
             continue;
 
         if ( ! addr->isAny() 
@@ -547,7 +648,7 @@ Address* PolicyCompiler::checkForZeroAddr::findHostWithNoInterfaces(RuleElement
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
 	FWObject *o= *i;
-	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	assert(o!=NULL);
         Host      *addr=Host::cast(o);
         if (addr!=NULL && addr->getFirstByType(Interface::TYPENAME)==NULL)
@@ -804,7 +905,7 @@ bool PolicyCompiler::MACFiltering::checkRuleElement(RuleElement *re)
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
         FWObject *o= *i;
-        if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         if (physAddress::isA(o)) 
         {
             lst.push_back(o);
@@ -865,7 +966,7 @@ string PolicyCompiler::debugPrintRule(Rule *r)
     RuleElementSrv *srvrel=rule->getSrv();
 
     string       iface_id = rule->getInterfaceId();
-//    Interface *rule_iface = fw_interfaces[iface_id];
+    Interface *rule_iface = fw_interfaces[iface_id];
 
     ostringstream str;
 
@@ -889,19 +990,21 @@ string PolicyCompiler::debugPrintRule(Rule *r)
 
         if (i1!=srcrel->end()) {
             FWObject *o=*i1;
-            if (FWReference::cast(o)!=NULL) o=getCachedObj(o->getStr("ref"));
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
             src+=o->getName();
+            src+="("+o->getId()+")";
         }
 
         if (i2!=dstrel->end()) {
             FWObject *o=*i2;
-            if (FWReference::cast(o)!=NULL) o=getCachedObj(o->getStr("ref"));
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
             dst+=o->getName();
+            dst+="("+o->getId()+")";
         }
 
         if (i3!=srvrel->end()) {
             FWObject *o=*i3;
-            if (FWReference::cast(o)!=NULL) o=getCachedObj(o->getStr("ref"));
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
             srv+=o->getName();
         }
 
@@ -920,6 +1023,7 @@ string PolicyCompiler::debugPrintRule(Rule *r)
         if (no==0) {
             str <<  setw(9)  << setfill(' ') << rule->getActionAsString().c_str();
             str <<  setw(9)  << setfill(' ') << rule->getDirectionAsString().c_str();
+            if (rule_iface!=NULL) str << " " << rule_iface->getName();
             if (rule->getLogging()) str << " LOG";
         } else
             str <<  setw(18)  << setfill(' ') << " ";
